@@ -18,17 +18,17 @@
 #include "accountmanager.h"
 #include "accountstate.h"
 #include "application.h"
+#include "common/restartmanager.h"
 #include "common/syncjournalfilerecord.h"
 #include "configfile.h"
-#include "creds/abstractcredentials.h"
 #include "folderman.h"
 #include "folderwizard/folderwizard.h"
 #include "gui/accountsettings.h"
 #include "gui/commonstrings.h"
+#include "gui/networkinformation.h"
 #include "guiutility.h"
 #include "libsync/theme.h"
 #include "logbrowser.h"
-#include "logger.h"
 #include "openfilemanager.h"
 #include "progressdispatcher.h"
 #include "settingsdialog.h"
@@ -43,10 +43,7 @@
 #include <QApplication>
 #include <QDesktopServices>
 #include <QDialog>
-#include <QDir>
 #include <QHBoxLayout>
-#include <QMessageBox>
-#include <QScreen>
 
 #ifdef Q_OS_WIN
 #include <qt_windows.h>
@@ -92,7 +89,8 @@ void setUpInitialSyncFolder(AccountStatePtr accountStatePtr, bool useVfs)
                     Utility::setupFavLink(localDir);
                     for (const auto *space : spaces) {
                         const QString name = space->displayName();
-                        const QString folderName = FolderMan::instance()->findGoodPathForNewSyncFolder(localDir, name);
+                        const QString folderName = FolderMan::instance()->findGoodPathForNewSyncFolder(
+                            localDir, name, FolderMan::NewFolderType::SpacesFolder, accountStatePtr->account()->uuid());
                         auto folder = addFolder(folderName, {}, QUrl(space->drive().getRoot().getWebDavUrl()), space->drive().getRoot().getId(), name);
                         folder->setPriority(space->priority());
                         // save the new priority
@@ -119,15 +117,14 @@ ownCloudGui::ownCloudGui(Application *parent)
     , _recentActionsMenu(nullptr)
     , _app(parent)
 {
-    // for the beginning, set the offline icon until the account was verified
-    _tray->setIcon(Theme::instance()->syncStateIcon(SyncResult::Status::Offline, true, false));
-
     connect(_tray, &QSystemTrayIcon::activated,
         this, &ownCloudGui::slotTrayClicked);
 
     setupActions();
     setupContextMenu();
 
+    // init systry
+    slotComputeOverallSyncStatus();
     _tray->show();
 
     ProgressDispatcher *pd = ProgressDispatcher::instance();
@@ -182,7 +179,7 @@ void ownCloudGui::slotTrayClicked(QSystemTrayIcon::ActivationReason reason)
         // However if the settings dialog is already visible but hidden
         // by other applications, this will bring it to the front.
         if (_settingsDialog->isVisible()) {
-            raiseDialog(_settingsDialog);
+            raise();
         }
 #else
         slotOpenSettingsDialog();
@@ -226,17 +223,18 @@ void ownCloudGui::slotAccountStateChanged()
 void ownCloudGui::slotTrayMessageIfServerUnsupported(Account *account)
 {
     if (account->serverSupportLevel() != Account::ServerSupportLevel::Supported) {
-        slotShowTrayMessage(
-            tr("Unsupported Server Version"),
+        slotShowTrayMessage(tr("Unsupported Server Version"),
             tr("The server on account %1 runs an unsupported version %2. "
                "Using this client with unsupported server versions is untested and "
                "potentially dangerous. Proceed at your own risk.")
-                .arg(account->displayName(), account->capabilities().status().versionString()));
+                .arg(account->displayNameWithHost(), account->capabilities().status().versionString()));
     }
 }
 
 void ownCloudGui::slotComputeOverallSyncStatus()
 {
+    auto getIcon = [this](const SyncResult &result) { return Theme::instance()->themeTrayIcon(result, contextMenuVisible()); };
+    auto getIconFromStatus = [getIcon](const SyncResult::Status &status) { return getIcon(SyncResult{status}); };
     bool allSignedOut = true;
     bool allPaused = true;
     bool allDisconnected = true;
@@ -267,7 +265,7 @@ void ownCloudGui::slotComputeOverallSyncStatus()
     }
 
     if (!problemAccounts.empty()) {
-        _tray->setIcon(Theme::instance()->syncStateIcon(SyncResult::Status::Offline, true, contextMenuVisible()));
+        _tray->setIcon(getIconFromStatus(SyncResult::Status::Offline));
         if (allDisconnected) {
             setStatusText(tr("Disconnected"));
         } else {
@@ -276,15 +274,15 @@ void ownCloudGui::slotComputeOverallSyncStatus()
 #ifdef Q_OS_WIN
         // Windows has a 128-char tray tooltip length limit.
         QStringList accountNames;
-        for (const auto &a : qAsConst(problemAccounts)) {
-            accountNames.append(a->account()->displayName());
+        for (const auto &a : std::as_const(problemAccounts)) {
+            accountNames.append(a->account()->displayNameWithHost());
         }
         _tray->setToolTip(tr("Disconnected from %1").arg(accountNames.join(QLatin1String(", "))));
 #else
         QStringList messages;
         messages.append(tr("Disconnected from accounts:"));
-        for (const auto &a : qAsConst(problemAccounts)) {
-            QString message = tr("Account %1").arg(a->account()->displayName());
+        for (const auto &a : std::as_const(problemAccounts)) {
+            QString message = tr("Account %1").arg(a->account()->displayNameWithHost());
             if (!a->connectionErrors().empty()) {
                 message += QLatin1String("\n") + a->connectionErrors().join(QLatin1String("\n"));
             }
@@ -296,12 +294,12 @@ void ownCloudGui::slotComputeOverallSyncStatus()
     }
 
     if (allSignedOut) {
-        _tray->setIcon(Theme::instance()->syncStateIcon(SyncResult::Status::Offline, true, contextMenuVisible()));
+        _tray->setIcon(getIconFromStatus(SyncResult::Status::Offline));
         _tray->setToolTip(tr("Please sign in"));
         setStatusText(tr("Signed out"));
         return;
     } else if (allPaused) {
-        _tray->setIcon(Theme::instance()->syncStateIcon(SyncResult::Paused, true, contextMenuVisible()));
+        _tray->setIcon(getIconFromStatus(SyncResult::Paused));
         _tray->setToolTip(tr("Account synchronization is disabled"));
         setStatusText(tr("Synchronization is paused"));
         return;
@@ -312,7 +310,7 @@ void ownCloudGui::slotComputeOverallSyncStatus()
 
     auto trayOverallStatusResult = FolderMan::trayOverallStatus(map);
 
-    const QIcon statusIcon = Theme::instance()->syncStateIcon(trayOverallStatusResult.overallStatus(), true, contextMenuVisible());
+    const QIcon statusIcon = getIcon(trayOverallStatusResult.overallStatus());
     _tray->setIcon(statusIcon);
 
     // create the tray blob message, check if we have an defined state
@@ -582,7 +580,7 @@ void ownCloudGui::updateContextMenu()
     slotRebuildRecentMenus();
 
     // We must call deleteLater because we might be called from the press in one of the actions.
-    for (auto *menu : qAsConst(_accountMenus)) {
+    for (auto *menu : std::as_const(_accountMenus)) {
         menu->deleteLater();
     }
     _accountMenus.clear();
@@ -595,12 +593,14 @@ void ownCloudGui::updateContextMenu()
     for (const auto &a : accountList) {
         if (a->isConnected()) {
             atLeastOneConnected = true;
+            break;
         }
     }
 
     for (auto *f : FolderMan::instance()->folders()) {
         if (f->syncPaused()) {
             atLeastOnePaused = true;
+            break;
         }
     }
 
@@ -611,17 +611,15 @@ void ownCloudGui::updateContextMenu()
         _contextMenu->addAction(tr("Create a new account"), this, &ownCloudGui::runNewAccountWizard);
     } else {
         if (atLeastOnePaused) {
-            _contextMenu->addAction(
-                tr("Resume synchronization"), this, [this] { setPauseOnAllFoldersHelper(AccountManager::instance()->accounts().values(), false); });
+            _contextMenu->addAction(tr("Resume synchronization"), this, [this] { setPauseOnAllFoldersHelper(AccountManager::instance()->accounts(), false); });
         } else {
-            _contextMenu->addAction(
-                tr("Stop synchronization"), this, [this] { setPauseOnAllFoldersHelper(AccountManager::instance()->accounts().values(), true); });
+            _contextMenu->addAction(tr("Stop synchronization"), this, [this] { setPauseOnAllFoldersHelper(AccountManager::instance()->accounts(), true); });
         }
         _contextMenu->addSeparator();
 
         // submenus for accounts
         for (const auto &account : accountList) {
-            QMenu *accountMenu = new QMenu(account->account()->displayName(), _contextMenu.data());
+            QMenu *accountMenu = new QMenu(account->account()->displayNameWithHost(), _contextMenu.data());
             _accountMenus.append(accountMenu);
             _contextMenu->addMenu(accountMenu);
 
@@ -635,10 +633,22 @@ void ownCloudGui::updateContextMenu()
     }
 
     if (_app->debugMode()) {
-        auto *crashMenu = _contextMenu->addMenu(QStringLiteral("Debug actions"));
-        crashMenu->addAction(QStringLiteral("Crash now - Div by zero"), _app, &Application::slotCrash);
-        crashMenu->addAction(QStringLiteral("Crash now - ENFORCE()"), _app, &Application::slotCrashEnforce);
-        crashMenu->addAction(QStringLiteral("Crash now - qFatal"), _app, &Application::slotCrashFatal);
+        auto *debugMenu = _contextMenu->addMenu(QStringLiteral("Debug actions"));
+        debugMenu->addAction(QStringLiteral("Crash if asserts enabled - OC_ENSURE"), _app, [] {
+            if (OC_ENSURE(false)) {
+                Q_UNREACHABLE();
+            }
+        });
+        debugMenu->addAction(QStringLiteral("Crash if asserts enabled - Q_ASSERT"), _app, [] { Q_ASSERT(false); });
+        debugMenu->addAction(QStringLiteral("Crash now - Utility::crash()"), _app, [] { Utility::crash(); });
+        debugMenu->addAction(QStringLiteral("Crash now - OC_ENFORCE()"), _app, [] { OC_ENFORCE(false); });
+        debugMenu->addAction(QStringLiteral("Crash now - qFatal"), _app, [] { qFatal("la Qt fatale"); });
+        debugMenu->addAction(QStringLiteral("Restart now"), _app, [] { RestartManager::requestRestart(); });
+        debugMenu->addSeparator();
+        auto captivePortalCheckbox = debugMenu->addAction(QStringLiteral("Behind Captive Portal"));
+        captivePortalCheckbox->setCheckable(true);
+        captivePortalCheckbox->setChecked(NetworkInformation::instance()->isForcedCaptivePortal());
+        connect(captivePortalCheckbox, &QAction::triggered, [](bool checked) { NetworkInformation::instance()->setForcedCaptivePortal(checked); });
     }
 
     _contextMenu->addSeparator();
@@ -681,8 +691,7 @@ void ownCloudGui::updateContextMenuNeeded()
 
 void ownCloudGui::slotShowTrayMessage(const QString &title, const QString &msg, const QIcon &icon)
 {
-    // SyncResult::Problem is returns the info icon
-    _tray->showMessage(title, msg, icon.isNull() ? Theme::instance()->syncStateIcon(SyncResult::Problem) : icon);
+    _tray->showMessage(title, msg, icon.isNull() ? Resources::getCoreIcon(QStringLiteral("states/information")) : icon);
 }
 
 void ownCloudGui::slotShowOptionalTrayMessage(const QString &title, const QString &msg, const QIcon &icon)
@@ -700,7 +709,7 @@ void ownCloudGui::slotShowOptionalTrayMessage(const QString &title, const QStrin
 void ownCloudGui::slotFolderOpenAction(Folder *f)
 {
     if (f) {
-        qCInfo(lcApplication) << "opening local url " << f->path();
+        qCInfo(lcApplication) << "opening local URL " << f->path();
         QUrl url = QUrl::fromLocalFile(f->path());
 
 #ifdef Q_OS_WIN
@@ -726,7 +735,7 @@ void ownCloudGui::slotRebuildRecentMenus()
 {
     _recentActionsMenu->clear();
     if (!_recentItemsActions.isEmpty()) {
-        for (auto *a : qAsConst(_recentItemsActions)) {
+        for (auto *a : std::as_const(_recentItemsActions)) {
             _recentActionsMenu->addAction(a);
         }
         _recentActionsMenu->addSeparator();
@@ -769,7 +778,7 @@ void ownCloudGui::slotUpdateProgress(Folder *folder, const ProgressInfo &progres
             msg = tr("Syncing %1 of %2  (%3 left)")
                       .arg(currentFile)
                       .arg(totalFileCount)
-                      .arg(Utility::durationToDescriptiveString2(progress.totalProgress().estimatedEta));
+                      .arg(Utility::durationToDescriptiveString2(std::chrono::milliseconds(progress.totalProgress().estimatedEta)));
         } else {
             msg = tr("Syncing %1 of %2")
                       .arg(currentFile)
@@ -781,7 +790,7 @@ void ownCloudGui::slotUpdateProgress(Folder *folder, const ProgressInfo &progres
         QString msg;
         if (progress.trustEta()) {
             msg = tr("Syncing %1 (%2 left)")
-                      .arg(totalSizeStr, Utility::durationToDescriptiveString2(progress.totalProgress().estimatedEta));
+                      .arg(totalSizeStr, Utility::durationToDescriptiveString2(std::chrono::milliseconds(progress.totalProgress().estimatedEta)));
         } else {
             msg = tr("Syncing %1")
                       .arg(totalSizeStr);
@@ -837,12 +846,18 @@ void ownCloudGui::runNewAccountWizard()
                     auto accountStatePtr = ocApp()->addNewAccount(newAccount);
                     accountStatePtr->setSettingUp(true);
 
+                    _settingsDialog->setCurrentAccount(accountStatePtr->account().data());
+
                     // ensure we are connected and fetch the capabilities
                     auto validator = new ConnectionValidator(accountStatePtr->account(), accountStatePtr->account().data());
 
                     QObject::connect(validator, &ConnectionValidator::connectionResult, accountStatePtr.data(),
                         [accountStatePtr, syncMode, dynamicRegistrationData](ConnectionValidator::Status status, const QStringList &) {
-                            if (OC_ENSURE(status == ConnectionValidator::Connected || status == ConnectionValidator::ServerVersionMismatch)) {
+                            switch (status) {
+                            // a server we no longer support but that might work
+                            case ConnectionValidator::ServerVersionMismatch:
+                                [[fallthrough]];
+                            case ConnectionValidator::Connected: {
                                 // saving once after adding makes sure the account is stored in the config in a working state
                                 // this is needed to ensure a consistent state in the config file upon unexpected terminations of the client
                                 // (for instance, when running from a debugger and stopping the process from there)
@@ -853,7 +868,7 @@ void ownCloudGui::runNewAccountWizard()
                                     OAuth::saveDynamicRegistrationDataForAccount(accountStatePtr->account(), dynamicRegistrationData);
                                 }
 
-                                // the account is now ready, emulate a normal account loading and emit that the credentials are ready
+                                // the account is now ready, emulate a normal account loading and Q_EMIT that the credentials are ready
                                 Q_EMIT accountStatePtr->account()->credentialsFetched();
 
                                 switch (syncMode) {
@@ -868,7 +883,6 @@ void ownCloudGui::runNewAccountWizard()
                                     Q_ASSERT(!accountStatePtr->account()->hasDefaultSyncRoot());
 
                                     auto *folderWizard = new FolderWizard(accountStatePtr, ocApp()->gui()->settingsDialog());
-                                    folderWizard->resize(ocApp()->gui()->settingsDialog()->sizeHintForChild());
                                     folderWizard->setAttribute(Qt::WA_DeleteOnClose);
 
                                     // TODO: duplication of AccountSettings
@@ -899,15 +913,22 @@ void ownCloudGui::runNewAccountWizard()
                                         accountStatePtr->setSettingUp(false);
                                     });
 
-                                    folderWizard->open();
-                                    ocApp()->gui()->raiseDialog(folderWizard);
-
+                                    ocApp()
+                                        ->gui()
+                                        ->settingsDialog()
+                                        ->accountSettings(accountStatePtr->account().get())
+                                        ->addModalLegacyDialog(folderWizard, AccountSettings::ModalWidgetSizePolicy::Expanding);
                                     break;
                                 }
                                 case OCC::Wizard::SyncMode::Invalid:
                                     Q_UNREACHABLE();
                                 }
                             }
+                            case ConnectionValidator::ClientUnsupported:
+                                break;
+                            default:
+                                Q_UNREACHABLE();
+                            }
                         });
 
 
@@ -921,9 +942,7 @@ void ownCloudGui::runNewAccountWizard()
             });
 
         // all we have to do is show the dialog...
-        _wizardController->window()->open();
-        // ... and bring it to the front
-        raiseDialog(_wizardController->window());
+        ocApp()->gui()->settingsDialog()->addModalWidget(_wizardController->window());
     }
 }
 
@@ -933,7 +952,7 @@ void ownCloudGui::setPauseOnAllFoldersHelper(const QList<AccountStatePtr> &accou
         if (accounts.contains(f->accountState())) {
             f->setSyncPaused(pause);
             if (pause) {
-                f->slotTerminateSync();
+                f->slotTerminateSync(tr("User paused sync for account '%1'").arg(f->accountState()->account()->displayNameWithHost()));
             }
         }
     }
@@ -941,13 +960,13 @@ void ownCloudGui::setPauseOnAllFoldersHelper(const QList<AccountStatePtr> &accou
 
 void ownCloudGui::slotShowSettings()
 {
-    raiseDialog(_settingsDialog);
+    raise();
 }
 
 void ownCloudGui::slotShowSyncProtocol()
 {
     slotShowSettings();
-    _settingsDialog->showActivityPage();
+    _settingsDialog->setCurrentPage(SettingsDialog::SettingsPage::Activity);
 }
 
 
@@ -964,8 +983,8 @@ void ownCloudGui::slotToggleLogBrowser()
 {
     auto logBrowser = new LogBrowser(settingsDialog());
     logBrowser->setAttribute(Qt::WA_DeleteOnClose);
+    ownCloudGui::raise();
     logBrowser->open();
-    raiseDialog(logBrowser);
 }
 
 void ownCloudGui::slotHelp()
@@ -973,25 +992,12 @@ void ownCloudGui::slotHelp()
     QDesktopServices::openUrl(QUrl(Theme::instance()->helpUrl()));
 }
 
-void ownCloudGui::raiseDialog(QWidget *raiseWidget)
+void ownCloudGui::raise()
 {
     auto window = ocApp()->gui()->settingsDialog();
-    auto *dialog = qobject_cast<QDialog *>(raiseWidget);
-    OC_ASSERT(window);
-    OC_ASSERT_X(!dialog || raiseWidget->parentWidget() == window, "raiseDialog should only be called with modal dialogs");
-    if (!window) {
-        return;
-    }
-    window->showNormal();
+    window->show();
     window->raise();
-    if (dialog && !dialog->isVisible()) {
-        dialog->open();
-    } else {
-        raiseWidget->showNormal();
-        raiseWidget->raise();
-    }
     window->activateWindow();
-    raiseWidget->activateWindow();
 
 #if defined(Q_OS_WIN)
     // Windows disallows raising a Window when you're not the active application.
@@ -1026,6 +1032,7 @@ void ownCloudGui::slotShowShareDialog(const QString &sharePath, const QString &l
             Utility::openBrowser(queryUrl, nullptr);
         });
     } else {
+        // oC10 code path
         const auto accountState = folder->accountState();
 
         SyncJournalFileRecord fileRecord;
@@ -1051,31 +1058,22 @@ void ownCloudGui::slotShowShareDialog(const QString &sharePath, const QString &l
             maxSharingPermissions = SharePermission(0);
         }
 
-
-        ShareDialog *w = nullptr;
-        if (_shareDialogs.contains(localPath) && _shareDialogs[localPath]) {
-            qCInfo(lcApplication) << "Raising share dialog" << sharePath << localPath;
-            w = _shareDialogs[localPath];
+        if (_shareDialog && _shareDialog->localPath() == localPath) {
+            qCInfo(lcApplication) << "A share dialog for this path already exists" << sharePath << localPath;
+            // There might be another modal dialog on top, but that is usually more important (e.g. a login dialog).
+            raise();
         } else {
-            qCInfo(lcApplication) << "Opening share dialog" << sharePath << localPath << maxSharingPermissions;
-            w = new ShareDialog(accountState, folder->webDavUrl(), sharePath, localPath, maxSharingPermissions, startPage, settingsDialog());
-            w->setAttribute(Qt::WA_DeleteOnClose, true);
-
-            _shareDialogs[localPath] = w;
-            connect(w, &QObject::destroyed, this, &ownCloudGui::slotRemoveDestroyedShareDialogs);
-        }
-        w->open();
-        raiseDialog(w);
-    }
-}
-
-void ownCloudGui::slotRemoveDestroyedShareDialogs()
-{
-    QMutableMapIterator<QString, QPointer<ShareDialog>> it(_shareDialogs);
-    while (it.hasNext()) {
-        it.next();
-        if (!it.value() || it.value() == sender()) {
-            it.remove();
+            qCInfo(lcApplication) << "Opening new share dialog" << sharePath << localPath << maxSharingPermissions;
+            if (_shareDialog) {
+                _shareDialog->close();
+            }
+            _shareDialog = new ShareDialog(accountState, folder->webDavUrl(), sharePath, localPath, maxSharingPermissions, startPage, settingsDialog());
+            _shareDialog->setAttribute(Qt::WA_DeleteOnClose, true);
+            ocApp()
+                ->gui()
+                ->settingsDialog()
+                ->accountSettings(accountState->account().get())
+                ->addModalLegacyDialog(_shareDialog, AccountSettings::ModalWidgetSizePolicy::Expanding);
         }
     }
 }
@@ -1085,9 +1083,8 @@ void ownCloudGui::slotAbout()
     if(!_aboutDialog) {
         _aboutDialog = new AboutDialog(_settingsDialog);
         _aboutDialog->setAttribute(Qt::WA_DeleteOnClose);
-        _aboutDialog->open();
+        ocApp()->gui()->settingsDialog()->addModalWidget(_aboutDialog);
     }
-    raiseDialog(_aboutDialog);
 }
 
 
