@@ -24,12 +24,10 @@
 #include "resources/resources.h"
 
 #include "QProgressIndicator.h"
-#include <QBuffer>
 #include <QClipboard>
 #include <QFileInfo>
-#include <QDesktopServices>
-#include <QMessageBox>
 #include <QMenu>
+#include <QMessageBox>
 #include <QToolButton>
 
 namespace OCC {
