@@ -26,7 +26,6 @@
 
 #include "csync_exclude.h"
 
-#include <QWidget>
 #include <QHeaderView>
 #include <QCoreApplication>
 #include <QDir>
@@ -74,8 +73,10 @@ const QString minChunkSizeC() { return QStringLiteral("minChunkSize"); }
 const QString maxChunkSizeC() { return QStringLiteral("maxChunkSize"); }
 const QString targetChunkUploadDurationC() { return QStringLiteral("targetChunkUploadDuration"); }
 const QString automaticLogDirC() { return QStringLiteral("logToTemporaryLogDir"); }
-const QString numberOfLogsToKeepC() { return QStringLiteral("numberOfLogsToKeep"); }
-const QString showExperimentalOptionsC() { return QStringLiteral("showExperimentalOptions"); }
+const QString numberOfLogsToKeepC()
+{
+    return QStringLiteral("numberOfLogsToKeep");
+}
 
 // The key `clientVersion` stores the version *with* build number of the config file. It is named
 // this way, because before 5.0, only the version *without* build number was stored.
@@ -95,9 +96,10 @@ const QString useDownloadLimitC() { return QStringLiteral("BWLimit/useDownloadLi
 const QString uploadLimitC() { return QStringLiteral("BWLimit/uploadLimit"); }
 const QString downloadLimitC() { return QStringLiteral("BWLimit/downloadLimit"); }
 
-const QString newBigFolderSizeLimitC() { return QStringLiteral("newBigFolderSizeLimit"); }
-const QString useNewBigFolderSizeLimitC() { return QStringLiteral("useNewBigFolderSizeLimit"); }
-const QString confirmExternalStorageC() { return QStringLiteral("confirmExternalStorage"); }
+const QString pauseSyncWhenMeteredC()
+{
+    return QStringLiteral("pauseWhenMetered");
+}
 const QString moveToTrashC() { return QStringLiteral("moveToTrash"); }
 
 const QString issuesWidgetFilterC()
@@ -726,33 +728,24 @@ void ConfigFile::setDownloadLimit(int kbytes)
     setValue(downloadLimitC(), kbytes);
 }
 
-QPair<bool, qint64> ConfigFile::newBigFolderSizeLimit() const
+bool ConfigFile::pauseSyncWhenMetered() const
 {
-    auto defaultValue = Theme::instance()->newBigFolderSizeLimit();
-    qint64 value = getValue(newBigFolderSizeLimitC(), QString(), defaultValue).toLongLong();
-    bool use = value >= 0 && getValue(useNewBigFolderSizeLimitC(), QString(), true).toBool();
-    return qMakePair(use, qMax<qint64>(0, value));
+    return getValue(pauseSyncWhenMeteredC(), {}, false).toBool();
 }
 
-void ConfigFile::setNewBigFolderSizeLimit(bool isChecked, qint64 mbytes)
+void ConfigFile::setPauseSyncWhenMetered(bool isChecked)
 {
-    setValue(newBigFolderSizeLimitC(), mbytes);
-    setValue(useNewBigFolderSizeLimitC(), isChecked);
-}
-
-bool ConfigFile::confirmExternalStorage() const
-{
-    return getValue(confirmExternalStorageC(), QString(), true).toBool();
-}
-
-void ConfigFile::setConfirmExternalStorage(bool isChecked)
-{
-    setValue(confirmExternalStorageC(), isChecked);
+    setValue(pauseSyncWhenMeteredC(), isChecked);
 }
 
 bool ConfigFile::moveToTrash() const
 {
-    return getValue(moveToTrashC(), QString(), false).toBool();
+    if (Theme::instance()->enableMoveToTrash()) {
+        auto defaultValue = Theme::instance()->moveToTrashDefaultValue();
+        return getValue(moveToTrashC(), QString(), defaultValue).toBool();
+    }
+
+    return false;
 }
 
 void ConfigFile::setMoveToTrash(bool isChecked)
@@ -849,12 +842,6 @@ bool ConfigFile::logHttp() const
     return settings.value(logHttpC(), false).toBool();
 }
 
-bool ConfigFile::showExperimentalOptions() const
-{
-    auto settings = makeQSettings();
-    return settings.value(showExperimentalOptionsC(), false).toBool();
-}
-
 QString ConfigFile::clientVersionWithBuildNumberString() const
 {
     auto settings = makeQSettings();
