@@ -20,9 +20,9 @@
 #include "creds/httpcredentials.h"
 #include "networkjobs/checkserverjobfactory.h"
 #include "networkjobs/fetchuserinfojobfactory.h"
+#include "resources/template.h"
 #include "theme.h"
 
-#include <QApplication>
 #include <QBuffer>
 #include <QDesktopServices>
 #include <QIcon>
@@ -32,8 +32,6 @@
 #include <QNetworkReply>
 #include <QPixmap>
 #include <QRandomGenerator>
-#include <QScopeGuard>
-#include <QTimer>
 
 using namespace std::chrono;
 using namespace std::chrono_literals;
@@ -44,16 +42,23 @@ Q_LOGGING_CATEGORY(lcOauth, "sync.credentials.oauth", QtInfoMsg)
 
 namespace {
 
-static const QString wellKnownPathC = QStringLiteral("/.well-known/openid-configuration");
+const QString wellKnownPathC = QStringLiteral("/.well-known/openid-configuration");
+
+const auto defaultOauthPromptValue()
+{
+    static const auto promptValue = [] {
+        OAuth::PromptValuesSupportedFlags out = OAuth::PromptValuesSupported::none;
+        // convert the legacy openIdConnectPrompt() to QFlags
+        for (const auto &x : Theme::instance()->openIdConnectPrompt().split(QLatin1Char(' '))) {
+            out |= Utility::stringToEnum<OAuth::PromptValuesSupported>(x);
+        }
+        return out;
+    }();
+    return promptValue;
+}
 
 QString renderHttpTemplate(const QString &title, const QString &content)
 {
-    const QString tmpl = [] {
-        QFile f(QStringLiteral(":/client/resources/oauth/oauth.html.in"));
-        OC_ASSERT(f.open(QFile::ReadOnly));
-        return QString::fromUtf8(f.readAll());
-    }();
-
     const QString icon = [] {
         const auto img = Theme::instance()->aboutIcon().pixmap(256).toImage();
         QByteArray out;
@@ -61,18 +66,19 @@ QString renderHttpTemplate(const QString &title, const QString &content)
         img.save(&buffer, "PNG");
         return QString::fromUtf8(out.toBase64());
     }();
-    return Utility::renderTemplate(tmpl, {
-                                             { QStringLiteral("TITLE"), title }, //
-                                             { QStringLiteral("CONTENT"), content }, //
-                                             { QStringLiteral("ICON"), icon }, //
-                                             { QStringLiteral("BACKGROUND_COLOR"), Theme::instance()->wizardHeaderBackgroundColor().name() }, //
-                                             { QStringLiteral("FONT_COLOR"), Theme::instance()->wizardHeaderTitleColor().name() } //
-                                         });
+    return Resources::Template::renderTemplateFromFile(QStringLiteral(":/client/resources/oauth/oauth.html.in"),
+        {
+            {QStringLiteral("TITLE"), title}, //
+            {QStringLiteral("CONTENT"), content}, //
+            {QStringLiteral("ICON"), icon}, //
+            {QStringLiteral("BACKGROUND_COLOR"), Theme::instance()->wizardHeaderBackgroundColor().name()}, //
+            {QStringLiteral("FONT_COLOR"), Theme::instance()->wizardHeaderTitleColor().name()} //
+        });
 }
 
 auto defaultTimeout()
 {
-    // as the oauth process can be interactive we don't want 5min of inactivity
+    // as the OAuth process can be interactive we don't want 5min of inactivity
     return qMin(30s, OCC::AbstractNetworkJob::httpTimeout);
 }
 
@@ -168,14 +174,19 @@ private:
         req.setHeader(QNetworkRequest::ContentTypeHeader, QByteArrayLiteral("application/json"));
         auto reply = _networkAccessManager->post(req, QJsonDocument(json).toJson());
         connect(reply, &QNetworkReply::finished, this, [reply, this] {
-            const auto data = reply->readAll();
-            QJsonParseError error {};
-            const auto json = QJsonDocument::fromJson(data, &error);
-            if (error.error == QJsonParseError::NoError) {
-                registerClientFinished(json.object().toVariantMap());
+            // https://datatracker.ietf.org/doc/html/rfc7591#section-3.2
+            if (reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt() == 201) {
+                const auto data = reply->readAll();
+                QJsonParseError error{};
+                const auto json = QJsonDocument::fromJson(data, &error);
+                if (error.error == QJsonParseError::NoError) {
+                    registerClientFinished(json.object().toVariantMap());
+                } else {
+                    qCWarning(lcOauth) << "Failed to register the client" << error.errorString() << data;
+                    Q_EMIT errorOccured(error.errorString());
+                }
             } else {
-                qCWarning(lcOauth) << "Failed to register the client" << error.errorString() << data;
-                Q_EMIT errorOccured(error.errorString());
+                Q_EMIT errorOccured(reply->errorString());
             }
         });
     }
@@ -239,6 +250,7 @@ OAuth::OAuth(const QUrl &serverUrl, const QString &davUser, QNetworkAccessManage
     , _clientId(Theme::instance()->oauthClientId())
     , _clientSecret(Theme::instance()->oauthClientSecret())
     , _redirectUrl(Theme::instance()->oauthLocalhost())
+    , _supportedPromptValues(defaultOauthPromptValue())
 {
 }
 
@@ -258,7 +270,7 @@ void OAuth::startAuthentication()
     }
     if (!_server.isListening()) {
         qCDebug(lcOauth) << "server is not listening";
-        emit result(Error, {});
+        Q_EMIT result(Error, {});
         return;
     }
 
@@ -267,31 +279,8 @@ void OAuth::startAuthentication()
     _state = generateRandomString(8);
 
     connect(this, &OAuth::fetchWellKnownFinished, this, [this] {
-        // this slightly complicated construct allows us to log case-specific messages
-        if (!Theme::instance()->oidcEnableDynamicRegistration()) {
-            qCDebug(lcOauth) << "dynamic registration disabled by theme";
-        } else if (!_registrationEndpoint.isValid()) {
-            qCDebug(lcOauth) << "registration endpoint not provided or empty:" << _registrationEndpoint;
-        } else {
-            auto registerJob = new RegisterClientJob(_networkAccessManager, _dynamicRegistrationData, _registrationEndpoint, this);
-            connect(registerJob, &RegisterClientJob::finished, this, [this](const QString &clientId, const QString &clientSecret, const QVariantMap &dynamicRegistrationData) {
-                qCDebug(lcOauth) << "client registration finished successfully";
-                _clientId = clientId;
-                _clientSecret = clientSecret;
-                Q_EMIT dynamicRegistrationDataReceived(dynamicRegistrationData);
-                Q_EMIT authorisationLinkChanged();
-            });
-            connect(registerJob, &RegisterClientJob::errorOccured, this, [this](const QString &error) {
-                qCWarning(lcOauth) << "Failed to dynamically register the client, try the default client id" << error;
-                Q_EMIT authorisationLinkChanged();
-            });
-            registerJob->start();
-
-            return;
-        }
-
-        // this should only run right now if one of the checks above failed
-        Q_EMIT authorisationLinkChanged();
+        connect(this, &AccountBasedOAuth::dynamicRegistrationDataReceived, this, &OAuth::authorisationLinkChanged);
+        updateDynamicRegistration();
     });
 
     fetchWellKnown();
@@ -382,7 +371,7 @@ void OAuth::startAuthentication()
                         qCWarning(lcOauth) << "Error when getting the accessToken" << errorReason;
                         httpReplyAndClose(socket, QStringLiteral("500 Internal Server Error"),
                             tr("Login Error"), tr("<h1>Login Error</h1><p>%1</p>").arg(errorReason));
-                        emit result(Error);
+                        Q_EMIT result(Error);
                         return;
                     }
 
@@ -399,7 +388,7 @@ void OAuth::startAuthentication()
                             if (!job->success()) {
                                 httpReplyAndClose(socket, QStringLiteral("500 Internal Server Error"), tr("Login Error"),
                                     tr("<h1>Login Error</h1><p>%1</p>").arg(job->errorMessage()));
-                                emit result(Error);
+                                Q_EMIT result(Error);
                             } else {
                                 auto fetchUserInfo = job->result().value<FetchUserInfoResult>();
 
@@ -415,7 +404,7 @@ void OAuth::startAuthentication()
                                                                "Please return to the %3 client and restart the authentication.</p>")
                                                                 .arg(fetchUserInfo.userName(), _davUser, Theme::instance()->appNameGUI());
                                     httpReplyAndClose(socket, QStringLiteral("403 Forbidden"), tr("Wrong user"), message);
-                                    emit result(Error);
+                                    Q_EMIT result(Error);
                                 } else {
                                     finalize(socket, accessToken, refreshToken, messageUrl);
                                 }
@@ -438,7 +427,7 @@ void OAuth::finalize(const QPointer<QTcpSocket> &socket, const QString &accessTo
     } else {
         httpReplyAndClose(socket, QStringLiteral("200 OK"), loginSuccessfulTitle, loginSuccessfulHtml);
     }
-    emit result(LoggedIn, accessToken, refreshToken);
+    Q_EMIT result(LoggedIn, accessToken, refreshToken);
 }
 
 QNetworkReply *OAuth::postTokenRequest(QUrlQuery &&queryItems)
@@ -458,14 +447,14 @@ QNetworkReply *OAuth::postTokenRequest(QUrlQuery &&queryItems)
     req.setHeader(QNetworkRequest::ContentTypeHeader, QStringLiteral("application/x-www-form-urlencoded; charset=UTF-8"));
     req.setAttribute(HttpCredentials::DontAddCredentialsAttribute, true);
 
-    queryItems.addQueryItem(QStringLiteral("scope"), Theme::instance()->openIdConnectScopes());
+    queryItems.addQueryItem(QStringLiteral("scope"), QString::fromUtf8(QUrl::toPercentEncoding(Theme::instance()->openIdConnectScopes())));
     req.setUrl(requestTokenUrl);
     return _networkAccessManager->post(req, queryItems.toString(QUrl::FullyEncoded).toUtf8());
 }
 
 QByteArray OAuth::generateRandomString(size_t size) const
 {
-    // TODO: do we need a varaible size?
+    // TODO: do we need a variable size?
     std::vector<quint32> buffer(size, 0);
     QRandomGenerator::global()->fillRange(buffer.data(), static_cast<qsizetype>(size));
     return QByteArray(reinterpret_cast<char *>(buffer.data()), static_cast<int>(size * sizeof(quint32))).toBase64(QByteArray::Base64UrlEncoding);
@@ -478,14 +467,12 @@ QUrl OAuth::authorisationLink() const
 
     const QByteArray code_challenge = QCryptographicHash::hash(_pkceCodeVerifier, QCryptographicHash::Sha256)
                                           .toBase64(QByteArray::Base64UrlEncoding | QByteArray::OmitTrailingEquals);
-    QUrlQuery query { { QStringLiteral("response_type"), QStringLiteral("code") },
-        { QStringLiteral("client_id"), _clientId },
-        { QStringLiteral("redirect_uri"), QStringLiteral("%1:%2").arg(_redirectUrl, QString::number(_server.serverPort())) },
-        { QStringLiteral("code_challenge"), QString::fromLatin1(code_challenge) },
-        { QStringLiteral("code_challenge_method"), QStringLiteral("S256") },
-        { QStringLiteral("scope"), Theme::instance()->openIdConnectScopes() },
-        { QStringLiteral("prompt"), Theme::instance()->openIdConnectPrompt() },
-        { QStringLiteral("state"), QString::fromUtf8(_state) } };
+    QUrlQuery query{{QStringLiteral("response_type"), QStringLiteral("code")}, {QStringLiteral("client_id"), _clientId},
+        {QStringLiteral("redirect_uri"), QStringLiteral("%1:%2").arg(_redirectUrl, QString::number(_server.serverPort()))},
+        {QStringLiteral("code_challenge"), QString::fromLatin1(code_challenge)}, {QStringLiteral("code_challenge_method"), QStringLiteral("S256")},
+        {QStringLiteral("scope"), QString::fromUtf8(QUrl::toPercentEncoding(Theme::instance()->openIdConnectScopes()))},
+        {QStringLiteral("prompt"), QString::fromUtf8(QUrl::toPercentEncoding(toString(_supportedPromptValues)))},
+        {QStringLiteral("state"), QString::fromUtf8(_state)}};
 
     if (!_davUser.isEmpty()) {
         const QString davUser = QString::fromUtf8(QUrl::toPercentEncoding(_davUser)); // Issue #7762;
@@ -511,7 +498,36 @@ QString OAuth::davUser() const
 
 void OAuth::saveDynamicRegistrationDataForAccount(const OCC::AccountPtr &accountPtr, const QVariantMap &dynamicRegistrationData)
 {
-    accountPtr->credentialManager()->set(dynamicRegistrationDataC(), dynamicRegistrationData);
+    if (!dynamicRegistrationData.isEmpty()) {
+        accountPtr->credentialManager()->set(dynamicRegistrationDataC(), dynamicRegistrationData);
+    }
+}
+
+void OAuth::updateDynamicRegistration()
+{
+    // this slightly complicated construct allows us to log case-specific messages
+    if (!Theme::instance()->oidcEnableDynamicRegistration()) {
+        qCDebug(lcOauth) << "dynamic registration disabled by theme";
+    } else if (!_registrationEndpoint.isValid()) {
+        qCDebug(lcOauth) << "registration endpoint not provided or empty:" << _registrationEndpoint.toString()
+                         << "we assume dynamic registration is not supported by the server";
+    } else {
+        auto registerJob = new RegisterClientJob(_networkAccessManager, _dynamicRegistrationData, _registrationEndpoint, this);
+        connect(registerJob, &RegisterClientJob::finished, this,
+            [this](const QString &clientId, const QString &clientSecret, const QVariantMap &dynamicRegistrationData) {
+                qCDebug(lcOauth) << "client registration finished successfully";
+                _clientId = clientId;
+                _clientSecret = clientSecret;
+                Q_EMIT dynamicRegistrationDataReceived(dynamicRegistrationData);
+            });
+        connect(registerJob, &RegisterClientJob::errorOccured, this, [this](const QString &error) {
+            qCWarning(lcOauth) << "Failed to dynamically register the client, try the default client id" << error;
+            Q_EMIT dynamicRegistrationDataReceived({});
+        });
+        registerJob->start();
+        return;
+    }
+    Q_EMIT dynamicRegistrationDataReceived({});
 }
 
 void OAuth::fetchWellKnown()
@@ -561,6 +577,16 @@ void OAuth::fetchWellKnown()
                 } else {
                     OC_ASSERT_X(false, qPrintable(QStringLiteral("Unsupported token_endpoint_auth_methods_supported: %1").arg(QDebug::toString(authMethods))));
                 }
+                const auto promptValuesSupported = data.value(QStringLiteral("prompt_values_supported")).toArray();
+                if (!promptValuesSupported.isEmpty()) {
+                    _supportedPromptValues = PromptValuesSupported::none;
+                    for (const auto &x : promptValuesSupported) {
+                        const auto flag = Utility::stringToEnum<PromptValuesSupported>(x.toString());
+                        // only use flags present in Theme::instance()->openIdConnectPrompt()
+                        if (flag & defaultOauthPromptValue())
+                            _supportedPromptValues |= flag;
+                    }
+                }
 
                 qCDebug(lcOauth) << "parsing .well-known reply successful, auth endpoint" << _authEndpoint
                                  << "and token endpoint" << _tokenEndpoint
@@ -617,7 +643,7 @@ void OAuth::openBrowser()
     if (!QDesktopServices::openUrl(authorisationLink())) {
         qCWarning(lcOauth) << "QDesktopServices::openUrl Failed";
         // We cannot open the browser, then we claim we don't support OAuth.
-        emit result(NotSupported, QString());
+        Q_EMIT result(NotSupported, QString());
     }
 }
 
@@ -736,33 +762,22 @@ void AccountBasedOAuth::refreshAuthentication(const QString &refreshToken)
         };
 
         connect(this, &OAuth::fetchWellKnownFinished, this, [refresh, this] {
-            // this slightly complicated construct allows us to log case-specific messages
-            if (!Theme::instance()->oidcEnableDynamicRegistration()) {
-                qCDebug(lcOauth) << "dynamic registration disabled by theme";
-            } else if (!_registrationEndpoint.isValid()) {
-                qCDebug(lcOauth) << "registration endpoint not provided or empty:" << _registrationEndpoint;
-            } else {
-                auto registerJob = new RegisterClientJob(_networkAccessManager, _dynamicRegistrationData, _registrationEndpoint, this);
-                connect(registerJob, &RegisterClientJob::finished, this, [this, refresh](const QString &clientId, const QString &clientSecret) {
-                    qCDebug(lcOauth) << "client registration finished successfully";
-                    _clientId = clientId;
-                    _clientSecret = clientSecret;
-                    refresh();
-                });
-                connect(registerJob, &RegisterClientJob::errorOccured, this, [refresh](const QString &error) {
-                    qCWarning(lcOauth) << "Failed to dynamically register the client, try the default client id" << error;
-                    refresh();
-                });
-                registerJob->start();
-
-                return;
-            }
-
-            // this should only run right now if one of the checks above failed
-            refresh();
+            connect(this, &AccountBasedOAuth::dynamicRegistrationDataReceived, this, refresh);
+            updateDynamicRegistration();
         });
         fetchWellKnown();
     });
 }
 
+QString OCC::toString(OAuth::PromptValuesSupportedFlags s)
+{
+    QStringList out;
+    for (auto k : {OAuth::PromptValuesSupported::consent, OAuth::PromptValuesSupported::select_account})
+        if (s & k) {
+            out += Utility::enumToString(k);
+        }
+    return out.join(QLatin1Char(' '));
+}
+
+
 #include "oauth.moc"
