@@ -15,18 +15,7 @@
 
 #include <QBuffer>
 #include <QCoreApplication>
-#include <QJsonDocument>
-#include <QLoggingCategory>
-#include <QMutex>
-#include <QNetworkAccessManager>
-#include <QNetworkReply>
 #include <QNetworkRequest>
-#include <QSslCipher>
-#include <QSslConfiguration>
-#include <QStack>
-#include <QStringList>
-#include <QTimer>
-#include <QXmlStreamReader>
 #include <QPainter>
 #include <QPainterPath>
 
@@ -51,10 +40,6 @@ RequestEtagJob::RequestEtagJob(AccountPtr account, const QUrl &rootUrl, const QS
 {
     setProperties({ QByteArrayLiteral("getetag") });
     connect(this, &PropfindJob::directoryListingIterated, this, [this](const QString &, const QMap<QString, QString> &value) {
-        if (reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt() != 207) {
-            Q_EMIT finishedWithError(reply());
-            return;
-        }
         _etag = Utility::normalizeEtag(value.value(QStringLiteral("getetag")));
         // the server returned a 207 but no etag, something is wrong
         if (!OC_ENSURE_NOT(_etag.isEmpty())) {
@@ -202,7 +187,7 @@ bool LsColXMLParser::parse(const QByteArray &xml, QHash<QString, qint64> *sizes,
                     if (currentHref.endsWith(QLatin1Char('/'))) {
                         currentHref.chop(1);
                     }
-                    emit directoryListingIterated(currentHref, currentHttp200Properties);
+                    Q_EMIT directoryListingIterated(currentHref, currentHttp200Properties);
                     currentHref.clear();
                     currentHttp200Properties.clear();
                 } else if (reader.name() == QLatin1String("propstat")) {
@@ -226,8 +211,8 @@ bool LsColXMLParser::parse(const QByteArray &xml, QHash<QString, qint64> *sizes,
         qCWarning(lcPropfindJob) << "ERROR no WebDAV response?" << xml;
         return false;
     } else {
-        emit directoryListingSubfolders(folders);
-        emit finishedWithoutError();
+        Q_EMIT directoryListingSubfolders(folders);
+        Q_EMIT finishedWithoutError();
     }
     return true;
 }
@@ -271,7 +256,7 @@ void PropfindJob::start()
                                     "<d:propfind xmlns:d=\"DAV:\">"
                                     "<d:prop>");
 
-        for (const QByteArray &prop : qAsConst(_properties)) {
+        for (const QByteArray &prop : std::as_const(_properties)) {
             const int colIdx = prop.lastIndexOf(':');
             if (colIdx >= 0) {
                 stream << QByteArrayLiteral("<") << prop.mid(colIdx + 1) << QByteArrayLiteral(" xmlns=\"") << prop.left(colIdx) << QByteArrayLiteral("\"/>");
@@ -298,41 +283,39 @@ void PropfindJob::finished()
     qCInfo(lcPropfindJob) << "LSCOL of" << reply()->request().url() << "FINISHED WITH STATUS"
                           << replyStatusString();
 
-    QString contentType = reply()->header(QNetworkRequest::ContentTypeHeader).toString();
-    int httpCode = reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
-    if (httpCode == 207 && contentType.contains(QLatin1String("application/xml; charset=utf-8"))) {
-        LsColXMLParser parser;
-        connect(&parser, &LsColXMLParser::directoryListingSubfolders,
-            this, &PropfindJob::directoryListingSubfolders);
-        connect(&parser, &LsColXMLParser::directoryListingIterated,
-            this, &PropfindJob::directoryListingIterated);
-        connect(&parser, &LsColXMLParser::finishedWithError,
-            this, &PropfindJob::finishedWithError);
-        connect(&parser, &LsColXMLParser::finishedWithoutError,
-            this, &PropfindJob::finishedWithoutError);
-        if (_depth == Depth::Zero) {
-            connect(&parser, &LsColXMLParser::directoryListingIterated, [&parser, counter = 0, this](const QString &name, const QMap<QString, QString> &) mutable {
-                counter++;
-                // With a depths of 0 we must receive only one listing
-                if (OC_ENSURE(counter == 1)) {
-                    disconnect(&parser, &LsColXMLParser::directoryListingIterated, this, &PropfindJob::directoryListingIterated);
-                } else {
-                    qCCritical(lcPropfindJob) << "Received superfluous directory listing for depth 0 propfind" << counter << "Path:" << name;
-                }
-            });
-        }
+    const QString contentType = reply()->header(QNetworkRequest::ContentTypeHeader).toString();
+    if (httpStatusCode() == 207) {
+        if (contentType.contains(QLatin1String("application/xml; charset=utf-8"))) {
+            LsColXMLParser parser;
+            connect(&parser, &LsColXMLParser::directoryListingSubfolders, this, &PropfindJob::directoryListingSubfolders);
+            connect(&parser, &LsColXMLParser::directoryListingIterated, this, &PropfindJob::directoryListingIterated);
+            connect(&parser, &LsColXMLParser::finishedWithoutError, this, &PropfindJob::finishedWithoutError);
+            if (_depth == Depth::Zero) {
+                connect(&parser, &LsColXMLParser::directoryListingIterated,
+                    [&parser, counter = 0, this](const QString &name, const QMap<QString, QString> &) mutable {
+                        counter++;
+                        // With a depths of 0 we must receive only one listing
+                        if (OC_ENSURE(counter == 1)) {
+                            disconnect(&parser, &LsColXMLParser::directoryListingIterated, this, &PropfindJob::directoryListingIterated);
+                        } else {
+                            qCCritical(lcPropfindJob) << "Received superfluous directory listing for depth 0 propfind" << counter << "Path:" << name;
+                        }
+                    });
+            }
 
-        QString expectedPath = reply()->request().url().path(); // something like "/owncloud/remote.php/webdav/folder"
-        if (!parser.parse(reply()->readAll(), &_sizes, expectedPath)) {
-            // XML parse error
-            emit finishedWithError(reply());
+            const QString expectedPath = reply()->request().url().path(); // something like "/owncloud/remote.php/webdav/folder"
+            if (!parser.parse(reply()->readAll(), &_sizes, expectedPath)) {
+                // XML parse error
+                Q_EMIT finishedWithError();
+            }
+        } else {
+            // wrong content type
+            qCWarning(lcPropfindJob) << "Unexpected ContentTypeHeader:" << contentType;
+            Q_EMIT finishedWithError();
         }
-    } else if (httpCode == 207) {
-        // wrong content type
-        emit finishedWithError(reply());
     } else {
         // wrong HTTP code or any other network error
-        emit finishedWithError(reply());
+        Q_EMIT finishedWithError();
     }
 }
 
@@ -389,7 +372,7 @@ void AvatarJob::finished()
             }
         }
     }
-    emit avatarPixmap(avImage);
+    Q_EMIT avatarPixmap(avImage);
 }
 
 /*********************************************************************************************/
@@ -407,7 +390,7 @@ void EntityExistsJob::start()
 
 void EntityExistsJob::finished()
 {
-    emit exists(reply());
+    Q_EMIT exists(reply());
 }
 
 /*********************************************************************************************/
@@ -437,13 +420,13 @@ void DetermineAuthTypeJob::finished()
 {
     auto authChallenge = reply()->rawHeader("WWW-Authenticate").toLower();
     auto result = AuthType::Basic;
-    if (authChallenge.contains("bearer ") || reply()->hasRawHeader("X-Oc-App-Password")) {
+    if (authChallenge.contains("bearer ") || reply()->hasRawHeader(QByteArrayLiteral("X-Oc-App-Password").toLower())) {
         result = AuthType::OAuth;
     } else if (authChallenge.isEmpty()) {
         qCWarning(lcDetermineAuthTypeJob) << "Did not receive WWW-Authenticate reply to auth-test PROPFIND";
     }
     qCInfo(lcDetermineAuthTypeJob) << "Auth type for" << _account->davUrl() << "is" << result;
-    emit this->authType(result);
+    Q_EMIT this->authType(result);
 }
 
 SimpleNetworkJob::SimpleNetworkJob(AccountPtr account, const QUrl &rootUrl, const QString &path, const QByteArray &verb, const QNetworkRequest &req, QObject *parent)
