@@ -18,7 +18,6 @@
 #include "networkjobs.h"
 #include "owncloudpropagator_p.h"
 #include "propagateremotedelete.h"
-#include "propagatorjobs.h"
 #include "syncengine.h"
 
 #include "common/asserts.h"
@@ -31,9 +30,7 @@
 
 #include <QDir>
 #include <QFileInfo>
-#include <QJsonDocument>
 #include <QJsonObject>
-#include <QNetworkAccessManager>
 
 #include <chrono>
 #include <cmath>
@@ -135,7 +132,7 @@ void PropagateUploadFileCommon::start()
     if (_item->_size > quotaGuess) {
         // Necessary for blacklisting logic
         _item->_httpErrorCode = 507;
-        emit propagator()->insufficientRemoteStorage();
+        Q_EMIT propagator()->insufficientRemoteStorage();
         done(SyncFileItem::DetailError, tr("Upload of %1 exceeds the quota for the folder").arg(Utility::octetsToString(_item->_size)));
         return;
     }
@@ -177,7 +174,7 @@ void PropagateUploadFileCommon::slotComputeContentChecksum()
 
     // we must be able to read the file
     if (FileSystem::isFileLocked(filePath, FileSystem::LockMode::SharedRead)) {
-        emit propagator()->seenLockedFile(filePath, FileSystem::LockMode::SharedRead);
+        Q_EMIT propagator()->seenLockedFile(filePath, FileSystem::LockMode::SharedRead);
         abortWithError(SyncFileItem::SoftError, tr("%1 the file is currently in use").arg(filePath));
         return;
     }
@@ -208,7 +205,7 @@ void PropagateUploadFileCommon::slotComputeTransmissionChecksum(CheckSums::Algor
     const QString filePath = propagator()->fullLocalPath(_item->_file);
     // we must be able to read the file
     if (FileSystem::isFileLocked(filePath, FileSystem::LockMode::SharedRead)) {
-        emit propagator()->seenLockedFile(filePath, FileSystem::LockMode::SharedRead);
+        Q_EMIT propagator()->seenLockedFile(filePath, FileSystem::LockMode::SharedRead);
         abortWithError(SyncFileItem::SoftError, tr("%1 the file is currently in use").arg(filePath));
         return;
     }
@@ -484,7 +481,7 @@ void PropagateUploadFileCommon::commonErrorHandling(AbstractNetworkJob *job)
         // Set up the error
         status = SyncFileItem::DetailError;
         errorString = tr("Upload of %1 exceeds the quota for the folder").arg(Utility::octetsToString(_item->_size));
-        emit propagator()->insufficientRemoteStorage();
+        Q_EMIT propagator()->insufficientRemoteStorage();
     } else if (_item->_httpErrorCode == 405) {
         status = SyncFileItem::FileIgnored;
         _item->setInstruction(CSYNC_INSTRUCTION_IGNORE);
@@ -497,7 +494,7 @@ void PropagateUploadFileCommon::commonErrorHandling(AbstractNetworkJob *job)
         status = SyncFileItem::FileIgnored;
         _item->setInstruction(CSYNC_INSTRUCTION_IGNORE);
         errorString = tr("File is sparse (contains large amounts of zeroes)");
-    }
+     }
 
     abortWithError(status, errorString);
 }
@@ -577,6 +574,8 @@ QMap<QByteArray, QByteArray> PropagateUploadFileCommon::headers()
 
 void PropagateUploadFileCommon::finalize()
 {
+    OC_ENFORCE(state() != Finished);
+
     // Update the quota, if known
     if (!_quotaUpdated) {
         auto quotaIt = propagator()->_folderQuota.find(QFileInfo(_item->_file).path());
@@ -588,13 +587,17 @@ void PropagateUploadFileCommon::finalize()
 
     if (_item->_remotePerm.isNull()) {
         qCWarning(lcPropagateUpload) << "PropagateUploadFileCommon::finalize: Missing permissions for" << propagator()->fullRemotePath(_item->_file);
-        auto permCheck = new PropfindJob(propagator()->account(), propagator()->webDavUrl(), propagator()->fullRemotePath(_item->_file), PropfindJob::Depth::Zero, this);
+        auto *permCheck =
+            new PropfindJob(propagator()->account(), propagator()->webDavUrl(), propagator()->fullRemotePath(_item->_file), PropfindJob::Depth::Zero, this);
         addChildJob(permCheck);
         permCheck->setProperties({ "http://owncloud.org/ns:permissions" });
         connect(permCheck, &PropfindJob::directoryListingIterated, this, [this](const QString &, const QMap<QString, QString> &map) {
             _item->_remotePerm = RemotePermissions::fromServerString(map.value(QStringLiteral("permissions")));
+            Q_ASSERT(!_item->_remotePerm.isNull());
             finalize();
         });
+        connect(permCheck, &PropfindJob::finishedWithError, this,
+            [this] { done(SyncFileItem::FatalError, tr("The server did not provide the file permissions")); });
         permCheck->start();
         return;
     }
@@ -631,13 +634,13 @@ void PropagateUploadFileCommon::abortNetworkJobs(
         return;
     _aborting = true;
 
-    // Count the number of jobs that need aborting, and emit the overall
+    // Count the number of jobs that need aborting, and Q_EMIT the overall
     // abort signal when they're all done.
     QSharedPointer<int> runningCount(new int(0));
     auto oneAbortFinished = [this, runningCount]() {
         (*runningCount)--;
         if (*runningCount == 0) {
-            emit this->abortFinished();
+            Q_EMIT this->abortFinished();
         }
     };
 
@@ -668,6 +671,6 @@ void PropagateUploadFileCommon::abortNetworkJobs(
     }
 
     if (*runningCount == 0 && abortType == AbortType::Asynchronous)
-        emit abortFinished();
+        Q_EMIT abortFinished();
 }
 }
